# Copyright (c) 2003 Nelson Minar <nelson@monkey.org>
# http://www.nelson.monkey.org/~nelson/weblog/tech/phone/

"Talk to a T616 via the serial port and make it do various things"

# import serial  # Requires pySerial http://pyserial.sourceforge.net/
import time, re, sys

contactParseRE=re.compile(r'''
  ^\+CPBR:\s
  (?P<pos>\d+),
  "(?P<num>[\dp*#+]+)",
  (?P<zone>\d+),
  "(?P<name>[^/]+)/(?P<type>\S)"$''',
                          re.VERBOSE)
def contactParse(s):
    """Parse a contact from the phone's format.
    Ex: '+CPBR: 4,"5551212",129,"Test/H"'
    Return as a tuple of strings - (position, number, zone, name, type)"""
    m = contactParseRE.match(s)
    if not m:
        raise "Could not parse contact string %s" % repr(s)
    return m.groups()

def contactUnparse(t):
    """Return a string like the phone's format
    Ex: 4,"5551212",129,"Test/H"
    """
    return '%s,"%s",%s,"%s/%s"' % t

def test():
    c1 = '+CPBR: 4,"5551212",129,"Test/H"'
    contact = contactParse(c1)
    assert ('4', '5551212', '129', 'Test', 'H') == contact
    assert c1[7:] == contactUnparse(contact)

    c2 = '+CPBR: 6,"9p9*#9+9",129,"Tdpt_/H"'
    contact = contactParse(c2)
    assert ('6', '9p9*#9+9', '129', 'Tdpt_', 'H') == contact
    assert c2[7:] == contactUnparse(contact)
        
class T616:
    """Class that encapsulates talking to a Sony Ericsson T616 and similar
    phones. Works via the serial port, sending AT commands to the phone
    and retrieving results.

    Methods starting with _ are low level communication methods.
    Other methods implement high-order functions

    Test environment:
      T616 firmware R1A054
      Belkin Bluetooth serial adapter
      ASCII only (phone does support Unicode, etc)
      Python 2.3.2 on Windows with win32all and pyserial 2.0

    More info:
    http://www.nelson.monkey.org/~nelson/weblog/tech/phone/
    http://www.ericsson.hu/mobilinternet/products/sh888/888_r1d.pdf
    """
    
    def __init__(self, port='COM4', timeout=1.0, debug=False):
        """Create a connection to the phone, clear the channel and test it.
        port: name of the serial port for pySerial to open
        timeout: default timeout for reads"""

        raise """This code is for demonstration purposes only. You must
be a Python programmer to make use of it. This code could potentially
destroy your phone. Understand it before using it!"""
    
        self.debug = debug
        
        # Open port
        self.ser = serial.Serial(port=port, baudrate=115200, timeout=timeout)
        # Turn off command echo
        self._sendCmd('ATE=0')
        # Read crud that's in buffers
        time.sleep(0.1)
        self._readWaiting()
        # And test the connection
        self._expect('AT', '\r\nOK\r\n')

    def _debug(self, s):
        if self.debug: sys.stderr.write(":: %s\n" % s)
        
    def _sendCmd(self, cmd):
        "Send a command to the phone. Ex: _sendCmd('AT')"
        self._debug("Sending %s" % cmd)
        self.ser.write(cmd)
        self.ser.write('\r\n')

    def _readWaiting(self):
        "Read whatever is waiting in the local serial buffer. No timeout."
        c = self.ser.inWaiting()
        return self.ser.read(c)

    def _readAll(self):
        "Read all input available. Relies on timeout."
        buf = []
        while 1:
            r = self.ser.read(512)
            if r:
                buf.append(r)
            else:
                return ''.join(buf)

    def _expect(self, cmd, expected):
        "Execute a command, then verify output. Relies on timeout."
        self._sendCmd(cmd)
        result = self.ser.read(len(expected))   # Rely on timeout here
        if not result == expected:
            raise "Expected %s but got %s" % (repr(expected), repr(result))

    def getTime(self):
        "Ask the phone what time it thinks it is"
        self._sendCmd('AT+CCLK?')
        r = self._readAll()
        return re.search(r'CCLK: "(.+)"', r).group(1)

    def getVersion(self):
        "Ask what version of firmware the phone has"
        self._sendCmd('AT+CGMR')
        return self._readAll()
    
    def selectMemory(self, type="ME"):
        "Select cell phone's memory (type can be one of ME or SM)"
        self._expect('AT+CPBS="%s"' % type, '\r\nOK\r\n')
        
    def readContacts(self):
        """Read all the contacts in the phone's memory.
        Returned as tuples."""
        # Ask the phone how many contacts it has
        # Result is like +CPBR: (1-510),80,180
        self._sendCmd('AT+CPBR=?')
        cpbrInfo = self._readAll()
        m = re.search(r'CPBR: \((\d+)-(\d+)\)', cpbrInfo)
        if not m:
            raise "Couldn't parse CPBR info %s" % repr(m)
        min, max = m.groups()

        # Ask the phone to dump all the contacts
        # Result is a list of lines like +CPBR: 2,"16506234316",129,"Nelson/W"
        # First and last three lines are not really contacts, so discard them
        self._sendCmd('AT+CPBR=%s,%s' % (min, max))
        contacts = self._readAll().split('\r\n')[1:-3]

        # Convert the strings to tuples
        r = []
        for c in contacts:
            try:
                r.append(contactParse(c))
            except Exception, e:
                # Continue past parse errors
                sys.stderr.write("%s\n" % e)
                
        return r

    def writeContact(self, c):
        """Write a contact tuple into the phone"""
        self._expect('AT+CPBW=%s' % contactUnparse(c), '\r\nOK\r\n')

    def close(self):
        "Close the phone connection politely"
        try:
            self._sendCmd('ATE=1')
        finally:    
            self.ser.close()

if __name__=='__main__':
    print "Running tests"
    test()

